#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using PrototypeHelperLibrary;
using MissileSwarmDodgingPrototype.World;
#endregion

namespace MissileSwarmDodgingPrototype
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        float aspectRatio;

        int missilesPerSalvo;
        double lastLaunchTime;

        PlayerShip playerModel;
        FollowCamera followCam;

        //SimpleBillboard testBillboard;

        public Game1()
        {
            this.IsFixedTimeStep = false;

            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            missilesPerSalvo = 3;
        }

        public ContentManager Content
        {
            get
            {
                return content;
            }
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            playerModel = new PlayerShip(this);
            this.Components.Add(playerModel);

            followCam = new FollowCamera(this);
            this.Components.Add(followCam);

            //testBillboard = new SimpleBillboard(this);
            //this.Components.Add(testBillboard);

            this.Components.Add(new KeyPressHelper(this, Keys.F1, new OnHelperKeyPress(ToggleDebugText)));
            //this.Components.Add(new KeyPressHelper(this, Keys.F2, new OnHelperKeyPress(ToggleDebugLine)));
            //this.Components.Add(new KeyPressHelper(this, Keys.LeftShift, new OnHelperKeyPress(SpawnSeekerMissile)));
            this.Components.Add(new KeyPressHelper(this, Keys.P, new OnHelperKeyPress(IncreaseMissilesPerSalvo)));
            this.Components.Add(new KeyPressHelper(this, Keys.O, new OnHelperKeyPress(DecreaseMissilesPerSalvo)));

            this.Components.Add(new KeyPressHelper(this, Keys.Up, new OnHelperKeyPress(DodgeUp)));
            this.Components.Add(new KeyPressHelper(this, Keys.Down, new OnHelperKeyPress(DodgeDown)));
            this.Components.Add(new KeyPressHelper(this, Keys.Left, new OnHelperKeyPress(DodgeLeft)));
            this.Components.Add(new KeyPressHelper(this, Keys.Right, new OnHelperKeyPress(DodgeRight)));

            base.Initialize();
        }

        public void ToggleDebugText()
        {
            DebugText.Visible = !DebugText.Visible;
        }

        public void ToggleDebugLine()
        {
            DebugLine.Visible = !DebugLine.Visible;
        }

        public void IncreaseMissilesPerSalvo()
        {
            if (missilesPerSalvo < 25)
                ++missilesPerSalvo;
        }

        public void DecreaseMissilesPerSalvo()
        {
            if (missilesPerSalvo > 1)
                --missilesPerSalvo;
        }

        public void DodgeUp()
        {
            playerModel.Dodge(SEEKER_LIST.UP);
        }

        public void DodgeDown()
        {
            playerModel.Dodge(SEEKER_LIST.DOWN);
        }

        public void DodgeLeft()
        {
            playerModel.Dodge(SEEKER_LIST.LEFT);
        }

        public void DodgeRight()
        {
            playerModel.Dodge(SEEKER_LIST.RIGHT);
        }

        public void SpawnSeekerMissile()
        {
            //Random randomNum = new Random((int)gameTime.TotalRealTime.Ticks);
            Random randomNum = new Random();

            for (int i = 0; i < missilesPerSalvo; ++i)
            {
                SeekerMissile missile = new SeekerMissile(this);
                missile.Position = new Vector3((float)randomNum.NextDouble() * 10000.0f - 5000.0f + playerModel.Position.X, (float)randomNum.NextDouble() * 10000.0f - 5000.0f + playerModel.Position.Y, playerModel.Position.Z - 50000.0f - (float)randomNum.NextDouble() *40000.0f);
                
                /*missile.Model.LoadModel(content, "Content/Models/p2_wedge");
                missile.Model.Rotation = MathHelper.Pi;
                missile.Model.Scale = 0.25f;
                missile.Model.Position = missile.Position;*/
                
                missile.Target = playerModel;
                this.Components.Add(missile);
            }
        }

        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                DebugText.LoadResources(graphics, content);
                DebugLine.LoadResources(graphics);

                playerModel.Model.LoadModel(content, "Content\\Models\\p1_wedge");

                //testBillboard.LoadTexture(content, "Content\\Billboard\\arrow_up", 0.5f, 0.5f );
                //testBillboard.Dimensions = new Vector2(1200.0f, 1200.0f);
                // TODO: Load any ResourceManagementMode.Automatic content
            }

            aspectRatio = graphics.GraphicsDevice.Viewport.Width / graphics.GraphicsDevice.Viewport.Height;
            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            DebugText.Output("Missile Swarm Prototype(Press ESC to exit)");
            DebugText.Output("Press F1 to toggle debut text.");
            DebugText.Output("Move ship with W, A, S, D keys. Dodge with arrow keys.");
            DebugText.Output("New missile salvo launches every second.");
            DebugText.Output("Missiles per salvo (Increase with P, Decrease with O): {0}\n", missilesPerSalvo);
            DebugText.Output("Launched Missile Count : {0}\n", SeekerMissile.LaunchCount);
            DebugText.Output("Active Missile Count    : {0}\n", SeekerMissile.ActiveCount);
            DebugText.Output("Hit Count               : {0}\n", playerModel.HitCount);
            DebugText.Output("Hit %                   : {0}%\n", (SeekerMissile.LaunchCount > 0 ) ? ( 100.0f * ((float)playerModel.HitCount / SeekerMissile.LaunchCount) ) : 0 );
            DebugText.Output("Failed dodges           : {0}\n", playerModel.FailedDodge);

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            //Vector3 steeringForce = Vector3.Zero;
            Vector3 steeringForce = playerModel.Velocity * -0.5f;
            steeringForce += new Vector3(0.0f, 0.0f, -250.0f);
            if (Keyboard.GetState().IsKeyDown(Keys.A))
            {
                steeringForce += new Vector3(-100.0f, 0.0f, 0.0f);
                //testBillboard.PatternX = 1;
            }
            else if (Keyboard.GetState().IsKeyDown(Keys.D))
            {
                steeringForce += new Vector3(100.0f, 0.0f, 0.0f);
                //testBillboard.PatternX = 0;
            }
            if (Keyboard.GetState().IsKeyDown(Keys.W))
            {
                steeringForce += new Vector3(0.0f, 100.0f, 0.0f);
            }
            else if (Keyboard.GetState().IsKeyDown(Keys.S))
            {
                steeringForce += new Vector3(0.0f, -100.0f, 0.0f);
            }
            playerModel.ApplySteeringForce(steeringForce);
            followCam.TargetPosition = playerModel.Position;

            //testBillboard.Position = playerModel.Position + new Vector3( 0.0f, 1000.0f, 0.0f );
            
            if (gameTime.TotalGameTime.TotalSeconds - lastLaunchTime > 1.0)
            {
                SpawnSeekerMissile();
                lastLaunchTime = gameTime.TotalGameTime.TotalSeconds;
            }

            base.Update(gameTime);
        }

        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 0.0f, 15000.0f);


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            //Matrix myView = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            Matrix myView = followCam.ViewMatrix;
            Matrix myProjection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 100000.0f);

            DebugLine.SetViewMatrix(myView);
            DebugLine.SetProjectionMatrix(myProjection);
            DebugLine.DrawLines(graphics);
            DebugText.DrawStrings();

            //followCam.Roll = playerModel.RollAmount;

            //graphics.GraphicsDevice.RenderState.AlphaTestEnable = true;
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = true;
            SimpleBillboard.SetProjectionMatrix(myProjection);
            SimpleBillboard.SetViewMatrix(myView);
            SimpleBillboard.SetCameraParameters(followCam.CameraPosition, followCam.CameraUp, followCam.CameraDirection);

            
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            graphics.GraphicsDevice.RenderState.DepthBufferEnable = true;
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = false;
            /*graphics.GraphicsDevice.RenderState.AlphaTestEnable = false;
            /*graphics.GraphicsDevice.SamplerStates[0].AddressU = TextureAddressMode.Wrap;
            graphics.GraphicsDevice.SamplerStates[0].AddressV = TextureAddressMode.Wrap;*/

            SimpleModel.SetViewMatrix(myView);
            SimpleModel.SetProjectionMatrix(myProjection);

            base.Draw(gameTime);
        }
    }
}
