#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using PrototypeHelperLibrary;
#endregion

namespace MissileSwarmDodgingPrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class FollowCamera : SteeringObject
    {
        protected Matrix m_ViewMatrix;
        protected Vector3 m_TargetPosition;
        protected Vector3 m_TargetOffset;

        protected Vector3 m_CurrentCameraPosition;
        protected float m_Roll = 0.0f;

        public FollowCamera(Game game)
            : base(game)
        {
            m_ViewMatrix = Matrix.Identity;
            m_TargetPosition = Vector3.Zero;
            m_TargetOffset = new Vector3(0.0f, 2500.0f, 10000.0f);

            m_CurrentCameraPosition = m_TargetPosition + m_TargetOffset;
        }

        public Matrix ViewMatrix
        {
            get
            {
                return m_ViewMatrix;
            }
        }

        public Vector3 TargetPosition
        {
            get
            {
                return m_TargetPosition;
            }
            set
            {
                m_TargetPosition = value;
            }
        }

        public Vector3 TargetOffset
        {
            get
            {
                return m_TargetOffset;
            }
            set
            {
                m_TargetOffset = value;
            }
        }

        public Vector3 CameraPosition
        {
            get
            {
                return m_CurrentCameraPosition;
            }
        }

        public Vector3 CameraUp
        {
            get
            {
                return Vector3.Up;
            }
        }

        public Vector3 CameraDirection
        {
            get
            {
                Vector3 dir = -m_TargetOffset;
                dir.Normalize();
                return dir;
            }
        }

        public float Roll
        {
            get
            {
                return m_Roll;
            }
            set
            {
                m_Roll = value;
            }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            //m_ViewMatrix = Matrix.CreateLookAt(m_TargetPosition + m_TargetOffset, m_TargetPosition, Vector3.Up);
            m_CurrentCameraPosition = m_CurrentCameraPosition + ((m_TargetPosition + m_TargetOffset) - m_CurrentCameraPosition) * 0.1f;
            //m_CurrentCameraPosition = m_TargetPosition + m_TargetOffset;
            m_ViewMatrix = Matrix.CreateLookAt(m_CurrentCameraPosition, m_TargetPosition, Vector3.Up) * Matrix.CreateFromAxisAngle(Vector3.Forward, m_Roll);
            //m_ViewMatrix = Matrix.CreateLookAt(m_TargetOffset * 2.0f, m_CurrentCameraPosition - Vector3.UnitZ, Vector3.Up);

            //DebugLine.Draw(m_CurrentCameraPosition, m_TargetPosition, Color.Red);

            base.Update(gameTime);
        }
    }
}


