#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using PrototypeHelperLibrary;
#endregion

namespace MissileSwarmDodgingPrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class MissileDodgeIndicator : Microsoft.Xna.Framework.GameComponent
    {
        protected const float WARNING_DISTANCE = 0.7f;
        protected const float DODGE_DISTANCE = 0.35f;

        protected PlayerShip m_PlayerShip;
        protected SeekerMissile m_Missile;
        protected SimpleBillboard m_StatusIndicator;
        protected float m_StartDistance;

        protected int m_CurrentFrame = 0;
        protected float m_ElapsedMillieconds = 0.0f;

        public MissileDodgeIndicator(Game game, PlayerShip playerShip, SeekerMissile missile, SEEKER_LIST direction)
            : base(game)
        {
            m_PlayerShip = playerShip;
            m_Missile = missile;

            m_StartDistance = Vector3.Distance(m_PlayerShip.Position, m_Missile.Position);

            m_StatusIndicator = new SimpleBillboard(game);
            game.Components.Add(m_StatusIndicator);

            switch (direction)
            {
                case SEEKER_LIST.UP:
                    m_StatusIndicator.LoadTexture(((Game1)game).Content, "Content\\Billboard\\arrow_up", 0.5f, 0.5f);
                    break;
                case SEEKER_LIST.DOWN:
                    m_StatusIndicator.LoadTexture(((Game1)game).Content, "Content\\Billboard\\arrow_down", 0.5f, 0.5f);
                    break;
                case SEEKER_LIST.LEFT:
                    m_StatusIndicator.LoadTexture(((Game1)game).Content, "Content\\Billboard\\arrow_left", 0.5f, 0.5f);
                    break;
                case SEEKER_LIST.RIGHT:
                    m_StatusIndicator.LoadTexture(((Game1)game).Content, "Content\\Billboard\\arrow_right", 0.5f, 0.5f);
                    break;
            }
            m_StatusIndicator.Dimensions = new Vector2(600.0f, 600.0f);

            game.Components.Add(this);
        }

        public SeekerMissile missile
        {
            get
            {
                return m_Missile;
            }
        }

        public void Destroy()
        {
            Game.Components.Remove(m_StatusIndicator);
            Game.Components.Remove(this);
        }

        public float WarningDistance
        {
            get
            {
                return m_StartDistance * WARNING_DISTANCE;
            }
        }


        public float DodgeDistance
        {
            get
            {
                return m_StartDistance * DODGE_DISTANCE;
            }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        public void SetPosition(Vector3 pos)
        {
            m_StatusIndicator.Position = pos;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            float missileDistance = Vector3.Distance(m_PlayerShip.Position, m_Missile.Position);
            float distanceRatio = missileDistance / m_StartDistance;

            if (distanceRatio > WARNING_DISTANCE)
            {
                m_StatusIndicator.PatternX = 1;
                m_StatusIndicator.PatternY = 0;
            }
            else if (distanceRatio > DODGE_DISTANCE)
            {
                m_StatusIndicator.PatternX = 0;
                m_StatusIndicator.PatternY = 1;
            }
            else 
            {
                m_ElapsedMillieconds += (float)gameTime.ElapsedGameTime.TotalMilliseconds;
                if (m_ElapsedMillieconds > 64.0f)
                {
                    m_ElapsedMillieconds -= 64.0f;
                    m_CurrentFrame = (m_CurrentFrame == 1) ? 0 : 1;
                }

                m_StatusIndicator.PatternX = m_CurrentFrame;
                m_StatusIndicator.PatternY = 1;
            }

            base.Update(gameTime);
        }
    }
}


