#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using PrototypeHelperLibrary;
#endregion

namespace MissileSwarmDodgingPrototype.World
{
    public enum SEEKER_LIST
    {
        UP = 0,
        DOWN = 1,
        LEFT = 2,
        RIGHT = 3,
        MAX = 4
    };

    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class PlayerShip : SteeringModel
    {
        protected Random randomNum = new Random();
        protected int m_HitCount = 0;
        protected int m_FailedDodge = 0;
        List<MissileDodgeIndicator>[] m_SeekerList;

        float m_RollAmount = 0.0f;
        float m_RollVelocity = 0.0f;

        public PlayerShip(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
            m_SeekerList = new List<MissileDodgeIndicator>[4];
            for( int i = 0; i < (int)SEEKER_LIST.MAX; ++i )
                m_SeekerList[i] = new List<MissileDodgeIndicator>();
        }

        public int HitCount
        {
            get
            {
                return m_HitCount;
            }
        }

        public int FailedDodge
        {
            get
            {
                return m_FailedDodge;
            }
        }

        public float RollAmount
        {
            get
            {
                return m_RollAmount;
            }
        }

        public bool AttemptHit(SeekerMissile missile)
        {
            if (!missile.Dodged)
            {
                ++m_HitCount;
                return true;
            }

            m_RollVelocity = (float)randomNum.NextDouble() * MathHelper.PiOver4 - MathHelper.PiOver4 * 0.5f;

            Vector3 steeringForce = (missile.Position - m_Position);
            steeringForce.Normalize();
            steeringForce = steeringForce * (m_MaxForce);
            steeringForce = -1.0f * (steeringForce - m_Velocity);
            ApplySteeringForce(steeringForce);

            return false;
        }

        public void Dodge(SEEKER_LIST direction)
        {
            foreach (MissileDodgeIndicator missileIndicator in m_SeekerList[(int)direction])
            {
                if (Vector3.Distance(m_Position, missileIndicator.missile.Position) < missileIndicator.WarningDistance)
                {
                    missileIndicator.missile.Dodged = true;
                    return;
                }
            }

            ++m_FailedDodge;
        }

        public void RegisterSeeker(SeekerMissile missile)
        {
            SEEKER_LIST directionList = (SEEKER_LIST)randomNum.Next(0, (int)SEEKER_LIST.MAX);
            m_SeekerList[(int)directionList].Add(new MissileDodgeIndicator( Game, this, missile, directionList));
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            m_RollAmount += m_RollVelocity;
            m_RollVelocity = m_RollVelocity * 0.5f;
            m_Model.RotationZ = m_RollAmount;

            List<MissileDodgeIndicator> deadMissileList = new List<MissileDodgeIndicator>();

            for( int i = 0; i < (int)SEEKER_LIST.MAX; ++i )
            {
                float missileCount = 1.0f;
                foreach (MissileDodgeIndicator missileStatus in m_SeekerList[i])
                {
                    switch( (SEEKER_LIST)i )
                    {
                        case SEEKER_LIST.UP:
                            missileStatus.SetPosition(this.Position + new Vector3(0.0f, 500.0f, 750.0f) * (float)missileCount);
                            break;
                        case SEEKER_LIST.DOWN:
                            missileStatus.SetPosition(this.Position + new Vector3(0.0f, -500.0f, 750.0f) * (float)missileCount);
                            break;
                        case SEEKER_LIST.LEFT:
                            missileStatus.SetPosition(this.Position + new Vector3(-500.0f, 0.0f, 750.0f) * (float)missileCount);
                            break;
                        case SEEKER_LIST.RIGHT:
                            missileStatus.SetPosition(this.Position + new Vector3(500.0f, 0.0f, 750.0f) * (float)missileCount);
                            break;
                    }
                    
                    if (missileStatus.missile.IsDestroyed || missileStatus.missile.Dodged)
                    {
                        deadMissileList.Add(missileStatus);
                    }
                    missileCount += 1.0f;
                }

                foreach( MissileDodgeIndicator deadMissile in deadMissileList )
                {
                    m_SeekerList[i].Remove(deadMissile);
                    deadMissile.Destroy();
                }
            }

            base.Update(gameTime);
        }
    }
}


