#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using PrototypeHelperLibrary;
#endregion

namespace MissileSwarmDodgingPrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SeekerMissile : SteeringModel
    {
        protected static int m_LaunchCount;
        protected static int m_ActiveCount;
        
        protected PlayerShip m_Target;
        protected float m_LifeTime;

        protected bool m_bDodged;
        protected List<Vector3> m_OldPositionList;
        protected float m_LastPointTime;
        protected float m_SecondsPerPoint;

        protected const float POINTS_PER_SECOND = 500.0f;

        public SeekerMissile(Game game)
            : base(game)
        {
            m_Target = null;
            m_MaxSpeed = 250.0f;
            m_LifeTime = 15.0f;

            m_bDodged = false;
            //m_bDodged = true;

            m_LastPointTime = 0.0f;
            m_SecondsPerPoint = m_LifeTime / POINTS_PER_SECOND;
            m_OldPositionList = new List<Vector3>((int)(m_LifeTime * POINTS_PER_SECOND));

            ++m_LaunchCount;
            ++m_ActiveCount;
        }

        public static int LaunchCount
        {
            get
            {
                return m_LaunchCount;
            }
        }

        public static int ActiveCount
        {
            get
            {
                return m_ActiveCount;
            }
        }

        public PlayerShip Target
        {
            get
            {
                return m_Target;
            }
            set
            {
                m_Target = value;
                if (m_Target != null)
                {
                    m_Target.RegisterSeeker(this);
                }
            }
        }

        public bool Dodged
        {
            get
            {
                return m_bDodged;
            }
            set
            {
                m_bDodged = value;
            }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            bool bTargetHit = false;
            if (m_Target != null)
            {
                Vector3 positionDifference = m_Position - m_Target.Position;
                if( positionDifference != Vector3.Zero )
                {
                    positionDifference.Normalize();
                    Vector3 desiredVelocity = positionDifference * m_MaxSpeed;
                    Vector3 steeringForce = desiredVelocity - m_Velocity;
                    ApplySteeringForce(-steeringForce);

                    if (Vector3.Distance(m_Position, m_Target.Position) < 1000.0f)
                    {
                        bTargetHit = m_Target.AttemptHit(this);
                    }
                }
            }

            float totalSeconds = (float)gameTime.TotalGameTime.TotalSeconds;
            float elapsedSeconds = (float)gameTime.ElapsedGameTime.TotalSeconds;

            m_LifeTime -= elapsedSeconds;

            if (totalSeconds - m_LastPointTime > m_SecondsPerPoint)
            {
                m_LastPointTime = totalSeconds;
                m_OldPositionList.Insert(0,m_Position);
            }

            Vector3 previousPoint = m_Position;
            int pointCount = 0;
            foreach (Vector3 pos in m_OldPositionList)
            {
                DebugLine.Draw(previousPoint, pos, (pointCount < POINTS_PER_SECOND * 0.01f) ? Color.Yellow : Color.Purple);
                previousPoint = pos;
                ++pointCount;
            }

            if (m_LifeTime < 0.0f || bTargetHit)
            {
                --m_ActiveCount;
                this.Destroy();
            }

            base.Update(gameTime);
        }
    }
}


