#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
#endregion

namespace MissileSwarmDodgingPrototype.World
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SteeringObject : Microsoft.Xna.Framework.GameComponent
    {
        protected float m_Mass;
        protected Vector3 m_Position;
        protected Vector3 m_Velocity;
        protected float m_MaxForce;
        protected float m_MaxSpeed;

        public SteeringObject(Game game)
            : base(game)
        {
            m_Mass = 10.0f;
            m_Position = Vector3.Zero;
            m_Velocity = Vector3.Zero;
            m_MaxForce = 200.0f;
            m_MaxSpeed = 100.0f;
        }

        public Vector3 Position
        {
            set
            {
                m_Position = value;
            }
            get
            {
                return m_Position;
            }
        }

        public Vector3 Velocity
        {
            get
            {
                return m_Velocity;
            }
        }

        protected Vector3 Truncate(Vector3 inVec, float maxVal)
        {
            Vector3 ret = inVec;
            if (ret.Length() > maxVal)
            {
                ret.Normalize();
                ret *= maxVal;
            }
            return ret;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        public void ApplySteeringForce(Vector3 steeringVector)
        {
            Vector3 steeringForce = steeringVector;
            steeringForce = Truncate(steeringForce, m_MaxForce);

            Vector3 acceleration = steeringForce / m_Mass;
            //acceleration = Truncate(acceleration, m_MaxSpeed);

            m_Velocity += acceleration;
            m_Velocity = Truncate(m_Velocity, m_MaxSpeed);

            //DebugLine.Draw(m_Position, (m_Position + steeringForce * 10.0f), Color.Green);
            //DebugLine.Draw(m_Position, (m_Position + m_Velocity * 10.0f), Color.Red);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            m_Position += m_Velocity;// *(float)gameTime.ElapsedGameTime.TotalSeconds;
            //m_Model.Position = new Vector3(m_Position, 0.0f);

            base.Update(gameTime);
        }
    }
}


